# This is an example program of how to use the Vaunix LSW DLL
# from python to control a LSW device
# Nov 15th 2023 -- Added the support for the changes to select between old and new switches

from ctypes import *

import ctypes
import os
import time
#vnx= cdll.VNX_switch64
vnx=cdll.LoadLibrary(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'VNX_switch64.dll'))
vnx.fnLSW_SetTestMode(False)        # Use actual devices
DeviceIDArray = c_int * 20
Devices = DeviceIDArray()           # This array will hold the list of device handles
                                    # returned by the DLL

# GetNumDevices will determine how many LSW devices are availible
numDevices = vnx.fnLSW_GetNumDevices()
print(str(numDevices), ' device(s) found')
if numDevices <= 0:
    print("Please connect a device and try again")
    exit()

# GetDevInfo generates a list, stored in the devices array, of
# every availible LSW device attached to the system
# GetDevInfo will return the number of device handles in the array
dev_info = vnx.fnLSW_GetDevInfo(Devices)
print('GetDevInfo returned', str(dev_info))

if numDevices > 1:
    print('Enter an integer between 1 and', numDevices, ': ', end = '')
    devid = Devices[int(input())-1]
else:
    devid = Devices[0]

cModelName = ctypes.create_string_buffer(32)
result = vnx.fnLSW_GetModelNameA(devid, ctypes.byref(cModelName));
modelname =  cModelName.value.decode("utf-8")
print("Model Name :", modelname)
# GetSerialNumber will return the devices serial number
ser_num = vnx.fnLSW_GetSerialNumber(devid)
print('Serial number:', str(ser_num))

#InitDevice will prepare the device for operation
init_dev = vnx.fnLSW_InitDevice(devid)
print('InitDevice returned', str(init_dev))

# Use GetNumSwitches to get the number of ports on the LSW device
numport = vnx.fnLSW_GetNumSwitches(devid)
print('There are', numport, 'ports on this LSW device')

# Use GetMaxSwitchDevices
if (modelname in ('LSW-802P4T', 'LSW-802PDT')):
    maxswitchdev = vnx.fnLSW_GetMaxSwitchDevices(devid)
    print('There are', maxswitchdev, 'devices on this LSW device')
 
# The GetSwitchSetting function returns which port is currently active
swindex = None
print('Read Switch port Status')
if (modelname in ('LSW-802P4T', 'LSW-802PDT')):
    if maxswitchdev > 1:
        print('Enter Switch index as an integer between 1 and', maxswitchdev, ': ', end = '')
        # This is what allows the user to enter their desired port
        swindex = int(input())
    else:
        swindex = 1
        
    result = vnx.fnLSW_GetSwitchRFoutput(devid, swindex)
    if result < 1 or result > numport:
        print('GetSwitchRFoutput returned an error', result)
    else:
        numport = vnx.fnLSW_GetNumSwitches(devid)
        print('Active RFout:', result)
else:
    # Use GetSwitchSetting
    switchsetting = vnx.fnLSW_GetSwitchSetting(devid)
    print('GetSwitchSetting returned ', switchsetting)

    # Use GetActiveSwitch
    activeswitch = vnx.fnLSW_GetActiveSwitch(devid)
    print('Active RFout:', activeswitch)
    
print('Set Switch RF output')
if (modelname in ('LSW-802P4T', 'LSW-802PDT')):
    print('Enter RFOutput index an integer between 1 and', numport, ': ', end= '')
    rfport = int(input())

    result = vnx.fnLSW_SetSwitchRFoutput(devid, swindex, rfport)
    if result != 0:
        print('SetSwitchRFoutput returned an error', result)
else:
    print('Select RF Switch output index an integer between 1 and', numport, ': ', end = '')
    switch = int(input())
    result = vnx.fnLSW_SetSwitch(devid, int(switch))
    if result != 0:
        print('SetSwitch returned an error', result)
        
# This function closes the device
# You should always close the devie when finished with it
closedev = vnx.fnLSW_CloseDevice(devid)
if closedev != 0:
    print('CloseDevice returned an error', closedev)
